const createError = require('http-errors');

//Unauthorized: https://tools.ietf.org/html/rfc6750#section-3.1

module.exports = {
  createInvalidRequestError(description = 'invalid token', message = 'invalid request') {
    return new createError(400, description, {
      headers: {
        'www-authentication': `Bearer realm="api", error="invalid_request", error_description="${message}"`
      },
      code: 'invalid_request'
    });
  },
  createInvalidTokenError(res, description = 'invalid token', message = 'invalid token') {
    res.set('www-authentication', `Bearer realm="api", error="invalid_token", error_description="${message}"`);
    return res.status(401).json({ success: false, code: 'invalid_token', description });
  },
  createInsufficientScopeError(res, scopes = false, description = 'insufficient scope', message = 'insufficient scope') {
    let header = `Bearer realm="api", error="invalid_token", error_description="${message}"`;

    if (scopes) {
      header += `, scope="${scopes.join(' ')}"`;
    }

    res.set('www-authentication', header);
    return res.status(403).json({ success: false, code: 'insufficient_scope', description });
  }
};
